--
-- KuhnFC313F
-- Specialization for KuhnFC313F functionality
--
-- Modschmiede / LS-Modsource
-- @author  Manuel Leithner
-- @date  10/08/09
--

KuhnFC313F = {};

function KuhnFC313F.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(FastCouplerAttachment, specializations);
end;

function KuhnFC313F:load(xmlFile)

	self.setHydraulicDirection = SpecializationUtil.callSpecializationsFunction("setHydraulicDirection");
	
	local hydraulicsCount = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.hydraulics#count"), 0);	
	self.hydraulics = {};	
	for i=1, hydraulicsCount do
		local hydraulicName = string.format("vehicle.hydraulics.hydraulic%d", i);		
		self.hydraulics[i] = {};		
		self.hydraulics[i].node = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#index"));
		self.hydraulics[i].punch = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#punch"));
		self.hydraulics[i].translationPunch = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#punchFixpoint"));
		self.hydraulics[i].fixPoint = Utils.indexToObject(self.components, getXMLString(xmlFile, hydraulicName .. "#fixpoint"));	
		local ax, ay, az;		
		if self.hydraulics[i].punch ~= nil then
			ax, ay, az = getWorldTranslation(self.hydraulics[i].punch);
		else
			ax, ay, az = getWorldTranslation(self.hydraulics[i].node);
		end;
		local bx, by, bz = getWorldTranslation(self.hydraulics[i].translationPunch);		
		self.hydraulics[i].punchDistance = Utils.vector3Length(ax-bx, ay-by, az-bz);
		self.hydraulics[i].doScale = Utils.getNoNil(getXMLBool(xmlFile, hydraulicName .. "#doScale"), false);
	end;
	
	self.hydraulicSafetyArm = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.safetyBars#hydraulic"));
	
	self.lift = {};
	self.lift.minTrans = {Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.lift#minLift"), 0)};
	self.lift.maxTrans = {Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.lift#maxLift"), 0)};
	self.lift.liftTime = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.lift#liftTime"), 3) * 1000;
	self.lift.detachPosition = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.lift#detachPosition"));
	local x,y,z = getTranslation(self.componentJoints[1].jointNode);
	self.lift.workingPosition = {x,y,z};
	self.lift.currentTransLimit = {0};
	self.lift.detachArm = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.lift#detachArm"));
	self.lift.detachArmRot = Utils.degToRad(getXMLFloat(xmlFile, "vehicle.lift#detachMaxRot"));
	self.lift.isDown = false;
	self.lift.isExpanded = false;
	
	x,y,z = getTranslation(self.lift.detachPosition);
	setTranslation(self.componentJoints[1].jointNode, x,y,z);
	setJointFrame(self.componentJoints[1].jointIndex, 0, self.componentJoints[1].jointNode);
	setJointTranslationLimit(self.componentJoints[1].jointIndex, 1, true, 0, 0);
	
	
	self.safetyBars = {};
	self.safetyBars.left = {};
	self.safetyBars.left.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.safetyBars#index1"));
	local z = Utils.degToRad(getXMLFloat(xmlFile, "vehicle.safetyBars#maxRot1"));
	self.safetyBars.left.maxRot = {0, 0, z};
	z = Utils.degToRad(getXMLFloat(xmlFile, "vehicle.safetyBars#minRot1"));
	self.safetyBars.left.minRot = {0, 0, z};
	self.safetyBars.right = {};
	self.safetyBars.right.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.safetyBars#index2"));
	z = Utils.degToRad(getXMLFloat(xmlFile, "vehicle.safetyBars#maxRot2"));
	self.safetyBars.right.maxRot = {0, 0, z};
	z = Utils.degToRad(getXMLFloat(xmlFile, "vehicle.safetyBars#minRot2"));
	self.safetyBars.right.minRot = {0, 0, z};
	self.safetyBars.rotTime = getXMLFloat(xmlFile, "vehicle.safetyBars#rotTime") * 1000;
	
	local jointTypeStr = getXMLString(xmlFile, "vehicle.attacherJoint#jointType");
	if jointTypeStr ~= nil then
		local jointType = Vehicle.jointTypeNameToInt[jointTypeStr];
		
		if jointType == nil then
			Vehicle.registerJointType(jointTypeStr);
			print("register Jointtype: " .. jointTypeStr);
			jointType = Vehicle.jointTypeNameToInt[jointTypeStr];
		end;
		
		if jointType ~= nil then
			self.attacherJoint.jointType = jointType;			
		end;
	end;
	
	
	self.particleSystems = {};
    local i=0;
    while true do
        local baseName = string.format("vehicle.particleSystems.particleSystem(%d)", i);

        local particleSystem = {};
        particleSystem.ps = {};
        local ps = Utils.loadParticleSystem(xmlFile, particleSystem.ps, baseName, self.components, false, nil, self.baseDirectory)
        if ps == nil then
            break;
        end;
        particleSystem.disableTime = 0;
        table.insert(self.particleSystems, particleSystem);
        i = i+1;
    end;
		
	self.groundReferenceThreshold = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.groundReferenceNode#threshold"), 0.2);
    self.groundReferenceNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.groundReferenceNode#index"));
    if self.groundReferenceNode == nil then
        self.groundReferenceNode = self.components[1].node;
    end;

    local mowerSound = getXMLString(xmlFile, "vehicle.mowerSound#file");
    if mowerSound ~= nil and mowerSound ~= "" then
        mowerSound = Utils.getFilename(mowerSound, self.baseDirectory); 
        self.mowerSound = createSample("mowerSound");
        self.mowerSoundEnabled = false;
        loadSample(self.mowerSound, mowerSound, false);
        self.mowerSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.mowerSound#pitchOffset"), 1);
        self.mowerSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.mowerSound#volume"), 1);
    end;

    local numCuttingAreas = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.cuttingAreas#count"), 0);
    for i=1, numCuttingAreas do
        local areanamei = string.format("vehicle.cuttingAreas.cuttingArea%d", i);
        self.cuttingAreas[i].foldMinLimit = Utils.getNoNil(getXMLFloat(xmlFile, areanamei .. "#foldMinLimit"), 0);
        self.cuttingAreas[i].foldMaxLimit = Utils.getNoNil(getXMLFloat(xmlFile, areanamei .. "#foldMaxLimit"), 1);
		self.cuttingAreas[i].particleSystemIndex = getXMLInt(xmlFile, areanamei .. "#particleSystemIndex");
    end;
	
	
	self.isDown = false;
	self.isExpanded = false;
	
	self.runs = 30;
	
	self.jointOrientation = false;
	self.rotationCopy = nil;
	self.attacherVehicleCopy  = nil;

end;

function KuhnFC313F:delete()
    if self.mowerSound ~= nil then
        delete(self.mowerSound);
    end;
end;

function KuhnFC313F:mouseEvent(posX, posY, isDown, isUp, button)
end;

function KuhnFC313F:keyEvent(unicode, sym, modifier, isDown)
end;

function KuhnFC313F:update(dt)	
	
	if self:getIsActive() then
		self.runs = 30;
		self.wasToFast = false;
		self.isExpanded = false;
		self.isLowered = false;
		
		local x, y, z = getRotation(self.safetyBars.left.node);
		if math.abs(z - self.safetyBars.left.minRot[3]) < 0.01 then
			self.isExpanded = true;
		end;
		
		if math.abs(self.lift.currentTransLimit[1]) > math.abs(self.lift.maxTrans[1] / 2) then
			self.isLowered = true;
		end;		
	end;
	
	if self.jointOrientation and self.fastCouplerJoint ~= nil then
		local joint = self.fastCouplerJoint;
		self.rotationCopy = {joint.minRot[1], joint.minRot[2], joint.minRot[3]};
		if joint.minRot[1] < math.rad(-20) then				
			joint.minRot[1] = math.rad(15);
		else
			joint.minRot[1] = math.rad(5);
		end;

		setRotation(self.fastCouplerJoint.jointTransform, math.rad(90), math.rad(-105), math.rad(-90));
		setJointFrame(self.fastCouplerJoint.jointIndex, 0, self.fastCouplerJoint.jointTransform);
		self.jointOrientation = false;
	end;
	
	
	if self:getIsActiveForInput() then
		if self.isExpanded then 
	        if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) then
	            self.isTurnedOn = not self.isTurnedOn;
	        end;
			
			if InputBinding.hasEvent(InputBinding.LOWER_IMPLEMENT) then
				self.lift.isDown = not self.lift.isDown;
			end;
		end;
		if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) and not self.isLowered and not self.isTurnedOn then
			self.lift.isExpanded = not self.lift.isExpanded;
		end;	
    end;

	if self:getIsActive() then			
	
		if self.fastCouplerJoint ~= nil and self.isExpanded then
			self.fastCouplerJoint.moveDown = false;
			self.attacherVehicleCopy.needsLowering = false;
		else
			self.attacherVehicleCopy.needsLowering = true;
		end;	

		for k,v in pairs(self.particleSystems) do
            if self.time > v.disableTime then
                Utils.setEmittingState(v.ps, false);
            end;
        end;
		
        if self.isTurnedOn then
            local toFast = false;
			if self.isLowered then
				toFast = self:doCheckSpeedLimit() and self.lastSpeed*3600 > 31;
			end;
            if not toFast and self.isLowered then
                local x,y,z = getWorldTranslation(self.groundReferenceNode);
				local foldAnimTime = self.foldAnimTime;
			
				local area = 0;
				local areaWheat = 0;
				local areaBarley = 0;
			
				for k, cuttingArea in pairs(self.cuttingAreas) do
				
					local x,y,z = getWorldTranslation(cuttingArea.start);
					local x1,y1,z1 = getWorldTranslation(cuttingArea.width);
					local x2,y2,z2 = getWorldTranslation(cuttingArea.height);
					
					if k == 1 then
						area = Utils.updateMeadowArea(x, z, x1, z1, x2, z2);
						areaWheat = Utils.cutFruitArea(FruitUtil.FRUITTYPE_WHEAT, x, z, x1, z1, x2, z2);
						areaBarley = Utils.cutFruitArea(FruitUtil.FRUITTYPE_BARLEY, x, z, x1, z1, x2, z2);
						
						if self.fastCouplerAttacher.movingDirection ~= -1 then
                            local ratio = g_currentMission.windrowCutLongRatio;
							local area = Utils.updateFruitCutLongArea(FruitUtil.FRUITTYPE_GRASS, x, z, x1, z1, x2, z2, 0);
                            area = area + Utils.updateFruitCutLongArea(FruitUtil.FRUITTYPE_GRASS, x, z, x1, z1, x2, z2, 0)*ratio;
						else
							if areaWheat > 0 then	
								local old, total = Utils.getFruitWindrowArea(FruitUtil.FRUITTYPE_WHEAT, x, z, x1, z1, x2, z2);
								local value = 1+math.floor(old / total + 0.7);
								value = math.min(value, g_currentMission.maxWindrowValue);
								Utils.updateFruitWindrowArea(FruitUtil.FRUITTYPE_WHEAT, x, z, x1, z1, x2, z2, value, true);	
							end;
							
							if areaBarley > 0 then
								local old, total = Utils.getFruitWindrowArea(FruitUtil.FRUITTYPE_BARLEY, x, z, x1, z1, x2, z2);
								local value = 1+math.floor(old / total + 0.7);
								value = math.min(value, g_currentMission.maxWindrowValue);
								Utils.updateFruitWindrowArea(FruitUtil.FRUITTYPE_BARLEY, x, z, x1, z1, x2, z2, value, true);							
							end;
						end;
						
						if g_currentMission.environment.lastRainScale <= 0.1 and g_currentMission.environment.timeSinceLastRain > 30 then
							if area > 0 or areaWheat > 0 or areaBarley > 0 then
								if cuttingArea.particleSystemIndex ~= nil then
									ps = self.particleSystems[cuttingArea.particleSystemIndex+1];
									if ps ~= nil then
										ps.disableTime = self.time + 20;
										Utils.setEmittingState(ps.ps, true);
									end;
								end;					
							end;
						end;
						
					else
						if self.fastCouplerAttacher.movingDirection ~= -1 then
							if area > 0 then
						        local lastArea = 0;
                                local realArea = 0;
							    local spray = Utils.getDensity(g_currentMission.terrainDetailId, g_currentMission.sprayChannel, x, z, x1, z1, x2, z2);
                                local multi = 1;
                                if spray > 0 then
                                    multi = 2;
                                end;
                                lastArea = lastArea + area*multi;
                                realArea = realArea + area/g_currentMission.maxFruitValue;
								local old, total = Utils.getFruitCutLongArea(FruitUtil.FRUITTYPE_GRASS, x, z, x1, z1, x2, z2);						
								local value = 1+math.floor(old / total + 0.02);
								value = math.min(value, g_currentMission.maxWindrowValue);
								Utils.updateFruitWindrowArea(FruitUtil.FRUITTYPE_GRASS, x, z, x1, z1, x2, z2, value, true, false);						
							end;
							
							if areaWheat > 0 then							
								local old, total = Utils.getFruitWindrowArea(FruitUtil.FRUITTYPE_WHEAT, x, z, x1, z1, x2, z2);
								local value = 1+math.floor(old / total + 0.7);
								value = math.min(value, g_currentMission.maxWindrowValue);
								Utils.updateFruitWindrowArea(FruitUtil.FRUITTYPE_WHEAT, x, z, x1, z1, x2, z2, value, true);	
							end;
							
							if areaBarley > 0 then
								local old, total = Utils.getFruitWindrowArea(FruitUtil.FRUITTYPE_BARLEY, x, z, x1, z1, x2, z2);
								local value = 1+math.floor(old / total + 0.7);
								value = math.min(value, g_currentMission.maxWindrowValue);
								Utils.updateFruitWindrowArea(FruitUtil.FRUITTYPE_BARLEY, x, z, x1, z1, x2, z2, value, true);							
							end;
						end;							
					end;
				end;
            end;

            if not self.mowerSoundEnabled and self:getIsActiveForSound() then
                setSamplePitch(self.mowerSound, self.mowerSoundPitchOffset);
                playSample(self.mowerSound, 0, self.mowerSoundVolume, 0);
                self.mowerSoundEnabled = true;
            end;

            self.wasToFast = toFast;

        else
            if self.mowerSoundEnabled then
                stopSample(self.mowerSound);
                self.mowerSoundEnabled = false;
            end;
        end;		
		
		
		
		local x, y, z = getRotation(self.safetyBars.left.node);	
		local rot = {x,y,z};
		local newRot = Utils.getMovedLimitedValues(rot, self.safetyBars.left.maxRot, self.safetyBars.left.minRot, 3, self.safetyBars.rotTime, dt, self.lift.isExpanded);
		setRotation(self.safetyBars.left.node, unpack(newRot));	
		x, y, z = getRotation(self.safetyBars.right.node);	
		rot = {x,y,z};
		newRot = Utils.getMovedLimitedValues(rot, self.safetyBars.right.maxRot, self.safetyBars.right.minRot, 3, self.safetyBars.rotTime, dt, self.lift.isExpanded);
		setRotation(self.safetyBars.right.node, unpack(newRot));		
	
		local newTransLimit = Utils.getMovedLimitedValues(self.lift.currentTransLimit, self.lift.maxTrans, self.lift.minTrans, 1, self.lift.liftTime, dt, not self.lift.isDown);
		if math.abs(newTransLimit[1] * 100 - self.lift.currentTransLimit[1]*100) > 0.001 then			
			setJointTranslationLimit(self.componentJoints[1].jointIndex, 1, true, -newTransLimit[1], 0);
		end;		
		self.lift.currentTransLimit = newTransLimit;		
	end;

	if self.runs > 0 then
		self:setHydraulicDirection();
		self.runs = self.runs - 1;
	end;
	
end;

function KuhnFC313F:draw()
	if self.isActive then
		if self.isExpanded then	
			if self.isLowered then
				g_currentMission:addHelpButtonText(string.format(g_i18n:getText("kuhnFC313F_lift_mower"), self.typeDesc), InputBinding.LOWER_IMPLEMENT);
			else
				g_currentMission:addHelpButtonText(string.format(g_i18n:getText("kuhnFC313F_lower_mower"), self.typeDesc), InputBinding.LOWER_IMPLEMENT);
			end;
			if self.isTurnedOn then
		        g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_off_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
		    else
		        g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_on_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
		    end;
			
--			if self.wasToFast then
--		        g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Cruise_control_levelN"), "2", InputBinding.getButtonKeyName(InputBinding.SPEED_LEVEL2)), 0.07+0.022, 0.019+0.029);
--		    end;
		end;
		
		if not self.isLowered and not self.isTurnedOn then
			if not self.lift.isExpanded then
				g_currentMission:addHelpButtonText(string.format(g_i18n:getText("kuhnFC313F_transport_off"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA2);
			else
				g_currentMission:addHelpButtonText(string.format(g_i18n:getText("kuhnFC313F_transport_on"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA2);
			end;
		end;
	end;
end;


function KuhnFC313F:onAttach(attacherVehicle)
	self.attacherVehicleCopy = attacherVehicle;
end;

function KuhnFC313F:onDetach()
	self.attacherVehicleCopy.needsLowering = true;
end;


function KuhnFC313F:setHydraulicDirection()
	for i=1, table.getn(self.hydraulics) do
		local ax, ay, az = getWorldTranslation(self.hydraulics[i].node);
		local bx, by, bz = getWorldTranslation(self.hydraulics[i].fixPoint);
		local x, y, z = worldDirectionToLocal(getParent(self.hydraulics[i].node), bx-ax, by-ay, bz-az);
		
		setDirection(self.hydraulics[i].node, x, y, z, 0, 1, 0);
		local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
		if self.hydraulics[i].doScale then
			local xScale, yScale, zScale = getScale(self.hydraulics[i].node);
			local newScale = yScale * (distance / self.hydraulics[i].punchDistance);
			setScale(self.hydraulics[i].node, 1, 1, newScale);
		else
			if self.hydraulics[i].punch ~= nil then
				setTranslation(self.hydraulics[i].punch, 0, 0, distance-self.hydraulics[i].punchDistance);
			end;
		end;
		
		if i == 1 then
			x,y,z = getRotation(self.hydraulics[i].node);
			setRotation(self.hydraulicSafetyArm, x, y, z);
		end;
	end;
end;


function KuhnFC313F:onFastCouplerAttach(attacherVehicle)
	self.jointOrientation = true;	
	setRotation(self.lift.detachArm, self.lift.detachArmRot,0,0);
	setTranslation(self.componentJoints[1].jointNode, unpack(self.lift.workingPosition));
	setJointFrame(self.componentJoints[1].jointIndex, 0, self.componentJoints[1].jointNode);
	self.isDown = false;
end;

function KuhnFC313F:onFastCouplerDetach()
	if self.fastCouplerJoint ~= nil then
		self.fastCouplerJoint.minRot = self.rotationCopy;
	end;
	
    if self.deactivateOnDetach then
        KuhnFC313F.onDeactivate(self);
    else
        KuhnFC313F.onDeactivateSounds(self)
    end;
	setRotation(self.lift.detachArm, 0,0,0);	

	local x,y,z = getTranslation(self.lift.detachPosition);
	setTranslation(self.componentJoints[1].jointNode, x,y,z);
	setJointFrame(self.componentJoints[1].jointIndex, 0, self.componentJoints[1].jointNode);
	setJointTranslationLimit(self.componentJoints[1].jointIndex, 1, true, 0, 0);
end;


function KuhnFC313F:onLeave()
    if self.deactivateOnLeave then
        KuhnFC313F.onDeactivate(self);
    else
        KuhnFC313F.onDeactivateSounds(self)
    end;
end;


function KuhnFC313F:onDeactivate()
    KuhnFC313F.onDeactivateSounds(self)
    for k,v in pairs(self.particleSystems) do
        Utils.setEmittingState(v.ps, false);
    end;
    self.isTurnedOn = false;
end;

function KuhnFC313F:onDeactivateSounds()
    if self.mowerSoundEnabled then
        stopSample(self.mowerSound);
        self.mowerSoundEnabled = false;
    end;
end;
